#include "xbasic_types.h"
#include "xio.h"
#include "font.h"
#include "depth_extract.h"

#define W 640
#define H 480
#define VGA_START 0x00800000
#define RED 0xE0
#define GREEN 0x1C
#define BLUE 0x03

#define FONT_WIDTH 8
#define FONT_HEIGHT 8
#define X_THRESHOLD 2000
#define Y_THRESHOLD 1000

extern void write_video(int start, int end, int line);
extern void track_object();
int last_x, last_y, last_rx, last_ry;
float last_depth, depth;

// Register addresses for SAA7114H configuration
unsigned char registers [] = {

  // Video decoder "generic" registers //
  0x01, 0x08, // Recommended setting
  0x02, 0xE4, // Analog input control 1 and input selection
  0x03, 0x10, // Analog input control 2
  0x04, 0x90, // Analog input control 3
  0x05, 0x90, // Analog input control 4
  0x06, 0xEB, // Horizontal Sync Start (delay)
  0x07, 0xE0, // Horizontal Sync Stop (delay)
  0x08, 0x59, //0x98, // 0xD8 // Sync control
  0x09, 0x40, // 0x45, // Luminance control
  0x0A, 0x80,
  0x0B, 0x44,
  0x0C, 0x40,
  0x0D, 0x00,
  0x0E, 0x89,
  0x0F, 0x2A, //0x24, // Chrominance gain
  0x10, 0x0E, //0x0C, // Chrominance control
  0x11, 0x00,
  0x12, 0x46, //0x00, // RT signal control
  0x13, 0x00,
  0x14, 0x00,
  0x15, 0x11,
  0x16, 0xFE,
  0x17, 0x40, //0x00, //0x40,

  0x18, 0x40,
  0x19, 0x80,
  0x1A, 0x00,
  0x1B, 0x00,
  0x1C, 0x00,
  0x1D, 0x00,
  0x1E, 0x00,
  0x30, 0x08, //0xBC, //0x08, // Audio clock stuff
  0x31, 0x08, //0xDF, //0x08, // Audio clock stuff
  0x32, 0x02,

  // Following 1 is added
  0x33, 0x00,

  0x34, 0xCD, //0x08, //0xCD, //0x08,
  0x35, 0xCC, //0x08, //0xCC, //0x08,
  0x36, 0x3A, //0x08, //0x3A, //0x08,

  // Following 1 is added
  0x37, 0x00,

  0x38, 0x03, //0x08, //0x03, //0x08,
  0x39, 0x10, //0x08, //0x10, //0x08,
  0x3A, 0x00, //0x03, //0x00, //0x03,


  // Following 5 are added
  0x3B, 0x00,
  0x3C, 0x00,
  0x3D, 0x00,
  0x3E, 0x00,
  0x3F, 0x00,

  0x40, 0x40, //0x00, //0x40, //0x00,
  0x41, 0xFF,
  0x42, 0xFF,
  0x43, 0xFF,
  0x44, 0xFF,
  0x45, 0xFF, 
  0x46, 0xFF, 
  0x47, 0xFF, 
  0x48, 0xFF,
  0x49, 0xFF,
  0x4A, 0xFF,
  0x4B, 0xFF,
  0x4C, 0xFF,
  0x4D, 0xFF,
  0x4E, 0xFF,
  0x4F, 0xFF,
  0x50, 0xFF,
  0x51, 0xFF,
  0x52, 0xFF,
  0x53, 0xFF,
  0x54, 0xFF,
  0x55, 0xFF,
  0x56, 0xFF,
  0x57, 0xFF,

  0x58, 0x40, //0x00, //0x40,
  0x59, 0x47,
  0x5A, 0x06,
  0x5B, 0x03, //0x83, //0x03,


  // Following 1 is added
  0x5C, 0x00,

  0x5D, 0x3E, //0x00, //0x3E, //0x00,
  0x5E, 0x00,

  // Following 1 is added
  0x5F, 0x00,


  0x80, 0x10, // 0x10:only Task A ;  0x30:both tasks.
  0x83, 0x01,
  0x84, 0xA0, //0xF0,
  0x85, 0x10, //0x00,
  0x86, 0x45,
  0x87, 0x01,
  0x88, 0xF0,

  // Task A Registers //
  0x90, 0x00, //0x00, //0x01,
  0x91, 0x08, //0x00,
  0x92, 0x10, //0x80,
  0x93, 0xC0, //0xC0,
  0x94, 0x10, //0x02,
  0x95, 0x00,
  0x96, 0xD0,
  0x97, 0x02,
  0x98, 0x0A,
  0x99, 0x00,
  0x9A, 0xF2, //0x00,
  0x9B, 0x00,
  0x9C, 0xD0, // Horizontal output window size upper bits \ 0xD002 = 720
  0x9D, 0x02, // Horizontal output window size lower bits /          by
  0x9E, 0xF0, // Vertical output window size upper bits  \  0xF000 = 240
  0x9F, 0x00, // Vertical output window size lower bits  /
  0xA0, 0x01,
  0xA1, 0x00,
  0xA2, 0x00,
  0xA4, 0x80,
  0xA5, 0x40,
  0xA6, 0x40,
  0xA8, 0x00,
  0xA9, 0x04,
  0xAA, 0x00,
  0xAC, 0x00,
  0xAD, 0x02,
  0xAE, 0x00,
  0xB0, 0x00,
  0xB1, 0x04,
  0xB2, 0x00,
  0xB3, 0x04, // 0x02
  0xB4, 0x00,
  0xB8, 0x00,

  // Following 3 were added
  0xB9, 0x00,
  0xBA, 0x00,
  0xBB, 0x00,

  0xBC, 0x00,

  // Following 3 were added
  0xBD, 0x00,
  0xBE, 0x00,
  0xBF, 0x00,

  /*
  // Task B Registers //
  0xC0, 0x08, //0x00,
  0xC1, 0x08, //0x00,
  0xC2, 0x10, //0x80,
  0xC3,	0xC0, 
  0xC4, 0x10, //0x02,
  0xC5, 0x00,
  0xC6, 0xD0, //0x01,
  0xC7, 0x02,
  0xC8, 0x0A,
  0xC9, 0x00,
  0xCA, 0xF2,
  0xCB, 0x00,
  0xCC, 0xD0,
  0xCD,	0x02,
  0xCE, 0xF0,
  0xCF, 0x00,
  0xD0, 0x01,
  0xD1, 0x00,
  0xD2, 0x00,
  0xD4, 0x80,
  0xD5, 0x40,
  0xD6, 0x40,
  0xD8, 0x00,
  0xD9,	0x04,
  0xDA, 0x00,
  0xDC, 0x00,
  0xDD, 0x02,
  0xDE, 0x00,
  0xE0, 0x00,
  0xE1, 0x04,
  0xE2, 0x00,
  0xE3, 0x04, //0x02
  0xE4, 0x00,
  0xE8,	0x00,
  0xE9, 0x00,
  0xEA, 0x00,
  0xEB, 0x00,
  0xEC,	0x00,
  0xED, 0x00,
  0xEE, 0x00,
  0xEF, 0x00,*/
				
  // Reset sequence. Extremelly needed!! 
  // Do not comment the following out! //
  0x88, 0xD8, //0xD0, //0xD8, 
  0x88, 0xF8, //0xF0, //0xF8,
  0xFF, 0xFF,};

int w = 0xFF;

char* itoa(int intval, char* char_array, int length) {
  int t, last_digit;
  int t_index, i;
  int t_intval;
  t_intval = intval;

    
  t_index = length-1;
  for (i=0;i<32;i++) {
    char_array[i]=' ';
  }

  while(t_intval > 9) {
    t = t_intval / 10;
    last_digit = t_intval-t*10;
    t_intval = t;
    char_array[t_index] = '0'+last_digit;
    t_index -= 1;
  }
  char_array[t_index] = t_intval+'0';
}

void i2c_delay()
{
    int i;
    for (i = 0; i < 1000; i++); 
}

// Write "level" to SCL //
void SCLw(int level)
{
    if (level == 0)
      w &= 0xDF;
    else
      w |= 0x2F;

    // Assert the clock on SCL //
    // according to level      //
    XIo_Out8(0xFEFF0200, w);
    i2c_delay();   
}

// Write "level" to SDA //
void SDAw(int level)
{
    if (level == 0)
      w &= 0x7F;
    else
      w |= 0x8F;

    // Assert the clock on SDA //
    // according to level      //
    XIo_Out8(0xFEFF0200, w); 
    i2c_delay();  
}

// Read from SDA //
int SDAr()
{
    int MSB = XIo_In8(0xFEFF0200);

    MSB = MSB >> 7;
    MSB &= 1;

    i2c_delay();
    return MSB;
}

// Tristate for SDA //
void SDAt(int rnw)
{
    if (rnw == 0)
      w &= 0xBF;
    else
      w |= 0x4F;

    // Assert the clock on SDA //
    // according to level      //
    XIo_Out8(0xFEFF0200, w);
    i2c_delay();
}

// Tristate for SCL //
void SCLt(int rnw)
{
    if (rnw == 0)
      w &= 0xEF;
    else
      w |= 0x1F;

    // Assert the clock on SDA //
    // according to level      //
    XIo_Out8(0xFEFF0200, w);
    i2c_delay();
}

// Send the start sequence
void send_start( void )
{
    SCLt(0);
    SDAt(0);
    SCLw(1);
    SDAw(0);
    SCLw(0);
}

// Send the restart sequence
// Needed for read register
void re_start( void )			/* This function must be entered with SDA High */
{
    SCLw(1);
    SDAw(0);
    SCLw(0);
}


// Send stop sequence
void send_stop( void )
{
    SCLw(0);
    SDAw(0);
    SCLw(1);
    SDAw(1);				/* Should leave with both lines high to indicate finish */
}

// Check acknowledge
int check_ack(void)
{
    int theresult;
    SDAt(1);
    SCLw(1);
    theresult=SDAr();
    SCLw(0);
    SDAw(1);				/* Set the output before it becomes active to eliminate spike */
    SDAt(0);
    return theresult;
}

// Send one bit
void send_bit(int x)
{
    x = x & 1; 
    SDAw(x);
    SCLw(1);
    SCLw(0);
}

// Send an entire bit
void send_byte(int byte)
{
    int i;
    for (i = 7; i >= 0; i--)
    { 
      send_bit(byte >> i);
    }
}

// Read a register from the video decoder
int read_register( int sub_address )
{
    int input = 0;
    int id;

    send_start();

    // Write slave address for SAA7114H is 43H //
    send_byte(0x42);

    check_ack();

    // Send the subaddress //
    send_byte(sub_address);

    check_ack();

    re_start();

    // Read address //
    send_byte(0x43);

    check_ack();
    
    SDAt(1);
    for( id=8 ; id>0 ; id=id-1 )
    {  
        input=input<<1;
        SCLw(1);
        input=input|SDAr();
        SCLw(0);
    }
    SDAw(1);   /* Set the output prior to enable to eliminate spike and make compatible with Restart */
    SDAt(0);
    SCLw(1);
    SCLw(0);
    send_stop();
    return input;
}

// Write a register into the video decoder
void write_register(int sub_address, int data)
{
    int i;

    // Start conditions //
    send_start();

    for (i = 0; i < 5; i++)
     i2c_delay();

    // Write slave address for SAA7114H is 42H //
    send_byte(0x42);

    check_ack();

    send_byte(sub_address);

    check_ack();

    send_byte(data);

    check_ack();

    send_stop();
}


inline void read_one_field()
{
  int line;
  int start, end;
  int line_section;
  Xuint32 current_level;

  //  if (!XIo_In32(0x018008FC))
    line = -1;
  //else
  //line = 239;

  while (1)
  {
    line = line + 1;

    // This variable indicates how much of //
    // the current line has  been  already //
    // written into the block RAMs         //
    current_level = 0x0001;
    
    for (line_section = 0; line_section < 4; line_section++)
    {
      // Wait for the current line to be  1/4,  1/2,  3/4 //
      // and full filled. The while then executes 4 times //
      while (!(XIo_In32(0x01803FFC) & current_level))
      {
	if (!XIo_In32(0x01802FFC))
	  break;
      }

      if (current_level == 0x01) {
	start = 0;
	end = 160;
      }
      else if (current_level == 0x02) {
	start = 160;
	end = 320;
      }
      else if (current_level == 0x04) {
	start = 320;
	end = 480;
      }
      else if (current_level == 0x08) {
	start = 480;
	end = 640;
      }
      
      if (!XIo_In32(0x01802FFC))
	break;
      
      write_video(start, end, line);
      
      current_level = current_level << 1;
      
      if (!XIo_In32(0x01802FFC))
	break;
    }
    
    if (!XIo_In32(0x01802FFC))
      break;
  }
}


int pos_x, pos_y;
void draw_byte(char x, int offset) {
  int i=0;
  int MASK = 1;
  for(i=0;i<8;i++) {
    if ((MASK&x)==0) {
      XIo_Out8(VGA_START + offset+(8-i), BLUE);
    }
    else {
      XIo_Out8(VGA_START + offset+(8-i), BLUE|GREEN|RED);
    }
    MASK=MASK<<1;
  }  
}

void write_c(char x) {
  int i=0;
  for(i=0;i<8;i++) {
    draw_byte(fontdata[x*8+i], pos_x*FONT_WIDTH+(pos_y+i)*W);
  }
}

void move_cursor() {
  pos_x += 1;
  if((pos_x+1)*FONT_WIDTH == W) {
    pos_y += FONT_HEIGHT;
    pos_x = 0;
  }
}

void move_cursor_back() {
  pos_x -= 1;
  if(pos_x*FONT_WIDTH < 0) {
    pos_x = 78;
    pos_y = pos_y - 8;
    if(pos_y<0){
      pos_y = 0;
    }
  }
}

void set_cursor(int x, int y) {
  pos_x = x;
  pos_y = y;
}


void write_char(char x) {
  switch(x) {
  case 8: //backspace
    write_c(' ');
    move_cursor_back();
    write_c(' ');
    break;
  case '\n': //new line \n
    write_c(' ');
    pos_y+=FONT_HEIGHT;
    break; 
  case '\r': //carriage return
    write_c(' ');
    pos_x = 0;
    break;
  case 20: //space
    move_cursor();
    break;
  case '\t'://tab
    write_char(' ');
    write_char(' ');
    write_char(' ');
    write_char(' ');
    break;
  default:
    write_c(x);
    move_cursor();
  }
}

void write_string(char* x, int length) {
  int i=0;
  for(i=0;i<length;i++) {
    write_char(x[i]);
  }
}
void shift_screen_up() {
  int x, y,temp;
  for(y=0;y<H-8;y++) {
    for(x=0;x<W/4;x++) {
      XIo_Out32(VGA_START+x*4+(y)*W, XIo_In32(VGA_START+x*4+(y+8)*W));
    }
  }
  for(y=H;y>=H-8;y--) {
    for(x=0;x<W;x++) {
      XIo_Out8(VGA_START + x+ y*W, BLUE|GREEN);
    }
  }
  pos_y = pos_y-8;
}

void draw_gui(int lx, int ly, int rx, int ry) {
  int i,j,q, tlx, trx, tval, temp_lx, average_x, average_y;
  char t[64];
  char depth_chars[3];
  temp_lx = lx;
  itoa(temp_lx, t, 3);
  tlx = lx;
  //trx = rx;
  trx = rx-320;
  
  tval = trx-tlx;

  //depth = tval+0.3;
  //xil_printf("tval %d tlx %d trx %d\t\r\n", tval, tlx, trx);  

  lx /= 8;
  //ly /= 8;
  rx /= 8;
  //ry /= 8;

  set_cursor(41, 410);


  write_string("Calculated 3D Coordinate:",25); 
  
  average_x = (tlx + trx) / 2;
  average_y = (ly + ry) / 2;
  
  itoa(average_x, t, 3);
  //xil_printf("tlx %d, trx, %d, average_x %d", temp_lx, trx, average_x);
  write_string(t, 3);
  itoa(average_y, t, 3);
  write_string(",", 1);
  write_string(t, 3);
  itoa(depth, t, 3);
  write_string(",", 1);
  write_string(t, 3);


  itoa(tval, depth_chars, 3);
  //for (i=0;i<64;i++) t[i]=' ';
  
  set_cursor(last_x, last_y);
  write_string(" ", 1);

  set_cursor(last_rx, last_ry);
  write_string(" ", 1);

  //for (i = 0; i < H*W; i++)
  //  XIo_Out8(VGA_START + i, 0);

  for(i=0;i<400;i++) {
    XIo_Out8(VGA_START + 320+i*W, RED);
  }
  for(j=0;j<W;j++) {
    XIo_Out8(VGA_START+j+400*W, RED);
  }

  for(j=0;j<W;j++) {
    XIo_Out8(VGA_START+j+9*W, RED);
  }

  set_cursor(10,1);
  write_string("Left Camera", 11);

  set_cursor(50,1);
  write_string("Right Camera", 12);
  
  set_cursor(1,410);
  write_string("Seperation Value: ",17);

  write_string(depth_chars,3);
  /*
    write_string("Depth (inches): ", 15);
    itoa((int)(last_depth), t, 3);

    write_string(t, 3);
  */
  set_cursor(1, 426);
  write_string("Light Source Coordinate:", 24);
  //print(t);
  //xil_printf("tlx %d \t\r\n", temp_lx);  
  itoa(temp_lx, t, 3);
  write_string(t, 3);
  write_string(":", 1);
  itoa(ly, t, 3);
  write_string(t, 3);

  set_cursor(41, 426);

  write_string("Light Source Coordinate:", 24);
  itoa(trx, t, 3);
  write_string(t, 3);
  write_string(":", 1);
  itoa(ry, t, 3);
  write_string(t, 3);
    
  last_x = lx, last_y=ly, last_rx=rx, last_ry=ry;
    

  set_cursor(25, 240);
  write_string("        ", 8);
  set_cursor(lx, ly);
  write_string("l", 1);

  set_cursor(55, 240);
  write_string("        ", 8);
  set_cursor(rx, ry);
  write_string("r", 1);
  //begin ruler part of code
  set_cursor(1,390);
  //write_string("Ruler", 6);

  for(j=0;j<W;j++) {
    XIo_Out8(VGA_START+j+440*W, RED);
  }
  for(j=0;j<W;j++) {
    XIo_Out8(VGA_START+j+450*W, RED);
  }
  for(j=0;j<W;j+=53) {
    if ((j/10)%2==0 && j<640) {
      itoa((j/53+1), depth_chars, 3);
      set_cursor(j/8, 455);
      write_string(depth_chars, 3);
    }          
  }


  //xvalues are divided by 8, do not divide y value by 8. stupid bug

  last_depth = depth;
  //xil_printf("\r\ndepth %d seperation: %d\t", (int) (depth*10), tval);
  depth = linear_depth(tval);
  //xil_printf("\r\n2depth %d seperation: %d\t", (int) (depth*10), tval);

  for(j=0;j<W;j+=1) {
    for(i=441;i<450;i++) {
      XIo_Out8(VGA_START + j +i*W, 0);	
    }
  }
  if(depth > 0 && depth < 12) {
    //xil_printf("\r\n HITHITxxxxxxxxxxxxxxxxxxxxxxxxx: %d\r\n", (int)(depth*10));
    for(i=440;i<450;i+=1) {
      XIo_Out8(VGA_START + (int)(depth*53 +i*W), RED);	
    }
  }
  else{
    //xil_printf("\r\n DEPTHxxxxxxxxxxxxxxxxxxxxxxxxx: %d\r\n", depth);
  }
  set_cursor(35, 465);
  write_string("inches", 6);
}

int main()
{ 
  int i, r1, r2, r3, r4 ,r5, counter,kash1,kash2;
  int lastlx, lastly, lastrx, lastry;

  //print("Hello World!\r\n");
  lastlx = lastly = lastrx = lastry = -1;

  microblaze_enable_icache();

  // Start the bus  protocol  by  sending //
  // a stop handshaking (SDA=1 and SCL=1) //
  send_stop();

  //print("Configuring video decoder...");

  i = 0;

  // Configure the video decoder SAA7114H //
  while (registers[i] != 0xFF) {
    write_register (registers[i], registers[i+1]);
    i+=2;
  }

  //print("Video decoder configured!\r\n");

  // Clear screen //
  for (i = 0; i < H*W; i++)
    XIo_Out8(VGA_START + i, 0);

  // Wait for a little bit
  for (i=0; i<10000;i++);

  while (1)
    {
      // Wait for the vertical synchronism
      while ((XIo_In32(0x01802FFC)));
      while (!(XIo_In32(0x01802FFC)));

      if (!XIo_In32(0x018008FC))
	{
	  while ((XIo_In32(0x01802FFC)));
	  while (!(XIo_In32(0x01802FFC)));
	}
      //read_one_field();
          
      //      for(counter=0;counter<5;counter++) {
      r1 = XIo_In32(0x01818FFC);
      xil_printf("lx %d\t", r1);
      r2 = XIo_In32(0x01828FFC);
      xil_printf("ly %d\t", r2);
      r3 = XIo_In32(0x01838FFC);
      xil_printf("rx %d\t", r3);
      r4 = XIo_In32(0x01814FFC);
      xil_printf("ry %d\t", r4);
      r5 = XIo_In32(0x01824FFC);
      xil_printf("r5 %d\r\n", r5);
      //track_object();
      //  }
      /*

      if (lastlx==-1) {
      lastlx = r1;
      }
      else {
      if (lastlx - r1 >0) {
      if ((lastlx - r1) < X_THRESHOLD) {
      lastlx = r1;
      }
      }
      else {
      if ((r1 - lastlx) < X_THRESHOLD) {
      lastlx = r1;
      }	    
      }
      }

      if (lastrx==-1) {
      lastrx = r3;
      }
      else {
      if (lastrx - r3 >0) {
      if ((lastrx - r3) < X_THRESHOLD) {
      lastrx = r3;
      }
      }
      else {
      if ((r3 - lastrx) < X_THRESHOLD) {
      lastrx = r3;
      }	    
      }
      }

      if (lastly==-1) {
      lastly = r2;
      }
      else {
      if (lastly - r2 >0) {
      if ((lastly - r2) < Y_THRESHOLD) {
      lastly = r2;
      }
      }
      else {
      if ((r2 - lastly) < Y_THRESHOLD) {
      lastly = r2;
      }	    
      }
      }

      if (lastry==-1) {
      lastry = r4;
      }
      else {
      if (lastry - r4 >0) {
      if ((lastry - r4) < Y_THRESHOLD) {
      lastry = r4;
      }
      }
      else {
      if ((r4 - lastry) < Y_THRESHOLD) {
      lastry = r4;
      }	    
      }
      }
      xil_printf("lx %d\t", lastlx);
      xil_printf("ly %d\t", lastly);
      xil_printf("rx %d\t", (W-lastrx+320));
      xil_printf("ry %d\t\r\n", lastry);

      draw_gui(lastlx,lastly, (W-lastrx+320), lastry);
      */      
      //xil_printf("lx %d\t", r1);
      //xil_printf("lx %d\t", r2);
      //xil_printf("lx %d\t", (W-r3+320));
      //xil_printf("lx %d\t\r\n", r4);

      draw_gui(r1, r2, (W-r3+320), r4);
      
    }
      

  //print("Goodbye\r\n");
  return 0;
}
