library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity opb_vidprocessor is  -- USER --
  generic
  (
    C_OPB_AWIDTH       : integer          := 32;
    C_OPB_DWIDTH       : integer          := 32;
    C_BASEADDR         : std_logic_vector := X"FEFF0000";
    C_HIGHADDR         : std_logic_vector := X"FEFF00FF");
  
  port
  (
    --Required OPB bus ports, do not add to or delete
    OPB_ABus     : in  std_logic_vector(0 to C_OPB_AWIDTH-1);
    OPB_BE       : in  std_logic_vector(0 to C_OPB_DWIDTH/8-1);
    OPB_Clk      : in  std_logic;
    OPB_DBus     : in  std_logic_vector(0 to C_OPB_DWIDTH-1);
    OPB_RNW      : in  std_logic;
    OPB_Rst      : in  std_logic;
    OPB_select   : in  std_logic;
    OPB_seqAddr  : in  std_logic;
    VIDPROC_DBus     : out std_logic_vector(0 to C_OPB_DWIDTH-1);
    VIDPROC_errAck   : out std_logic;
    VIDPROC_retry    : out std_logic;
    VIDPROC_toutSup  : out std_logic;
    VIDPROC_xferAck  : out std_logic;

    -- USER --
    ICLK             : in std_logic;
    IDQ              : in std_logic;
    ITRDY            : out std_logic;
    ITRI             : out std_logic;
    IGPH             : in std_logic;
    IGPV             : in std_logic;
    IGP0             : in std_logic;
    IGP1             : in std_logic;
    IPD              : in std_logic_vector(7 downto 0);
    HPD              : in std_logic_vector(7 downto 0)
    --VID_I2C_SCL      : inout std_logic;
    --VID_I2C_SDA      : inout std_logic    
  );
 
end entity opb_vidprocessor; --USER-- 

-------------------------------------------------------------------------------
-- architecture
-------------------------------------------------------------------------------

architecture imp of opb_vidprocessor is --USER-- 

  component comparator
     port (
       clk              : in std_logic;
       load             : in std_logic;
       reset            : in std_logic;
       input_x          : in std_logic_vector(10 downto 0);
       input_y          : in std_logic_vector(15 downto 0);
       input_lum      : in std_logic_vector(7 downto 0);
       max_lum       : out std_logic_vector (7 downto 0);
       max_x            : out std_logic_vector(10 downto 0);
       max_y            : out std_logic_vector(15 downto 0)
       );
   end component;

-- added
   component IBUF
     port (
      I : in std_logic;
       O:  out std_logic
       );
   end component;
-- added
  component BUFG
    port (
      I : in  std_logic;
      O : out std_logic);
  end component;

  component FD
    port (
      C : in std_logic;
      D : in std_logic;
      Q : out std_logic);
  end component;

attribute iob : string;
attribute iob of FD : component is "true";
-- component IOBUF_F_12
--  port (
--     O : out STD_ULOGIC;
--     IO : inout STD_ULOGIC;
--     I : in STD_ULOGIC;
--     T : in STD_ULOGIC);
-- end component;
 
  signal wdata : std_logic_vector(0 to 7); 
  signal rdata : std_logic_vector(0 to 15);
  signal rnw  : std_logic;
  signal cs, xfer : std_logic;
  signal q0,q1 : std_logic;
  signal i2c_din : std_logic;

  --added
  signal buff_op_lum_l               : std_logic_vector(7 downto 0);
  signal buff_x_l                    : std_logic_vector(10 downto 0);
  signal buff_y_l                    : std_logic_vector(15 downto 0);
  signal buff_x_r                    : std_logic_vector(10 downto 0);
  signal buff_y_r                    : std_logic_vector(15 downto 0);
  
  
  signal op_lum_l               : std_logic_vector(7 downto 0);
  signal x_l                    : std_logic_vector(10 downto 0);
  signal y_l                    : std_logic_vector(15 downto 0);
  signal op_lum_r               : std_logic_vector(7 downto 0);
  signal x_r                    : std_logic_vector(10 downto 0);
  signal y_r                    : std_logic_vector(15 downto 0);
   
  signal count_x                : std_logic_vector(10 downto 0);
  signal count_y                : std_logic_vector(15 downto 0);
  signal load_left              : std_logic;
  signal load_right             : std_logic;
  
  signal load                   : std_logic_vector(1 downto 0);
  signal buf_IPD                : std_logic_vector(7 downto 0);
  signal buf_IDQ                : std_logic;
  signal buf_ICLK0              : std_logic;
  signal buf_ICLK               : std_logic;
  signal buf_IGPH               : std_logic;
  signal buf_IGPV               : std_logic;
  signal latch_IGPV             : std_logic;
  signal latch_IGPH             : std_logic;
  signal latch_IDQ              : std_logic;
  signal latch_IPD              : std_logic_vector(7 downto 0);
  signal frame_counter          : std_logic_vector(1 downto 0);
  
begin


-- sda_pad : IOBUF_F_12 port map (
--   I => wdata(0),
--   IO => VID_I2C_SDA,
--   O =>  i2c_din,
--   T => wdata(1)
-- );

-- scl_pad : IOBUF_F_12 port map (
--   I => wdata(2),
--   IO => VID_I2C_SCL,
--   O => open,
--   T => wdata(3)
-- );
  
-- Chip select, memory mapped. 
process (OPB_select, OPB_ABus)
begin

  if(OPB_select='1' and OPB_ABus(0 to 23)=C_BASEADDR(0 to 23)) then
    cs <= '1'; else
    cs <= '0';
  end if;  
end process;


-- number of pixel(horizontal)
process(buf_ICLK,OPB_Rst)
  begin
     if OPB_Rst='1' then
       count_x  <= "00000000000";
     elsif buf_ICLK'event and buf_ICLK ='1' then
       if latch_IDQ ='1' then              
         if latch_IGPH ='0' then
           count_x <= "00000000000";
         else
           count_x <= count_x + 1;
         end if; 
       end if;
     end if;
end process;

-- number of lines
process (buf_ICLK, OPB_Rst)
begin
   if OPB_Rst='1' then
     count_y <= X"0000";
   elsif buf_ICLK'event and buf_ICLK ='1' then
     if latch_IGPV ='0' then
       count_y <= X"0000";
     elsif latch_IGPH ='1' and count_x =719 then
       count_y <= count_y + 1;
    end if;
  end if;
end process;

-- give the frame ID
process (buf_ICLK, OPB_Rst)
begin
  if OPB_Rst ='1' then
    frame_counter <= "00";
  elsif buf_ICLK'event and buf_ICLK ='1' then
    if count_y = 239 and count_x =719 then
      frame_counter <= frame_counter+1;
    end if;
  end if;
end process;

-- load left or right for one frame
process (buf_ICLK)
begin  -- process
  if buf_ICLK'event and buf_ICLK = '1' then
    if ( frame_counter(0) = '0') then
      if count_x < 360 then
        load_left <= '1';
        load_right <= '0';
      else
        load_left <= '0';
        load_right <= '1';
      end if;
    end if;
  end if;
end process;

--added output comparator result buffer
    process (frame_counter)
    begin 
      --if buf_ICLK'event and buf_ICLK='1' then
        if frame_counter(0) = '1' then
          buff_x_l <= x_l;
          buff_y_l <= y_l;
          buff_x_r <= x_r;
          buff_y_r <= y_r;
          buff_op_lum_l <= op_lum_l;
        end if;
      --end if;
    end process;

-- read data
process (OPB_Clk,OPB_Rst)
begin
  if (OPB_Rst='1') then
    wdata <= "11111111";
    rdata <= X"0000";
  elsif OPB_Clk'event and OPB_Clk = '1' then
    rnw <= OPB_RNW;
    if (q1 = '0' and q0 = '1' and rnw='0') then
       wdata <= OPB_DBus(0 to 7);
    end if;
    if (q1='1') then
       rdata <= X"0000";
    elsif (q1='0' and q0='1' and rnw = '1') then
      if OPB_ABus(24 to 31) = X"00" then
        rdata <= "00000" & buff_x_l;
      elsif OPB_ABus(24 to 31) = X"04"  then
        rdata <= buff_y_l;
      elsif OPB_ABus(24 to 31) = X"08" then
        rdata <= "00000" & buff_x_r;
      elsif OPB_ABus(24 to 31) = X"0C" then
        rdata <= buff_y_r;
      elsif OPB_ABus(24 to 31) = X"10" then
        rdata <= X"00" & op_lum_l;
      else
        rdata <= X"0000";    
      end if;
      -- rdata <= "00000011"; --i2c_din & "0000000";
    end if;    
  end if;
end process;


process (OPB_Clk,OPB_Rst)
begin
  if (OPB_Rst = '1') then
    q0 <= '0';
    q1 <= '0';
  elsif OPB_Clk'event and OPB_Clk='1' then
    q1 <= not q1 and q0;
    q0 <= not q1 and not q0 and cs;
  end if;
end process;

xfer <= q1;

VIDPROC_xferAck <= xfer;

VIDPROC_DBus <=  X"0000" & rdata;

VIDPROC_errAck <= '0';

VIDPROC_retry <= '0';

VIDPROC_toutSup <= '0';


 comparator_left : comparator port map (
    clk         => buf_ICLK,
    load        => load_left,
    reset       => frame_counter(0),
    input_x     => count_x,
    input_y     => count_y,
    input_lum   => latch_IPD,
    max_lum     => op_lum_l,
    max_x       => x_l,
    max_y       => y_l
  );

  comparator_right : comparator port map (
    clk         => buf_ICLK,
    load        => load_right,
    reset       => frame_counter(0),
    input_x     => count_x,
    input_y     => count_y,
    input_lum   => latch_IPD,
    max_lum     => op_lum_r,
    max_x       => x_r,
    max_y       => y_r
  );

     bufigph : IBUF port map (
       I => IGPH,
       O => buf_IGPH
     );
     bufigpv : IBUF port map (
       I => IGPV,
       O => buf_IGPV
     ); 
     bufipd0 : IBUF port map (
       I => IPD(0),
       O => buf_IPD(0)
     );
     bufipd1 : IBUF port map (
       I => IPD(1),
       O => buf_IPD(1)
     );
     bufipd2 : IBUF port map (
       I => IPD(2),
       O => buf_IPD(2)
     );
     bufipd3 : IBUF port map (
       I => IPD(3),
       O => buf_IPD(3)
     );
     bufipd4 : IBUF port map (
       I => IPD(4),
       O => buf_IPD(4)
     );
     bufipd5 : IBUF port map (
       I => IPD(5),
       O => buf_IPD(5)
     );
     bufipd6 : IBUF port map (
       I => IPD(6),
       O => buf_IPD(6)
     );
     bufipd7 : IBUF port map (
       I => IPD(7),
       O => buf_IPD(7)
     );
     bufidq : IBUF port map (
       I => IDQ,
       O => buf_IDQ
     );
     buficlk : IBUF port map (
       I => ICLK,
       O => buf_ICLK0
     );
     buficlk2 : BUFG port map(
       I => buf_ICLK0,
       O => buf_ICLK
     );
     igpv_pinlatch : FD port map (
       C => buf_ICLK,
       D => buf_IGPV,
       Q => latch_IGPV
     );
     igph_pinlatch : FD port map (
       C => buf_ICLK,
       D => buf_IGPH,
       Q => latch_IGPH
     );
     idq_pinlatch : FD port map (
       C => buf_ICLK,
       D => buf_IDQ,
       Q => latch_IDQ
     );  
     latchipd0 : FD port map (
       C => buf_ICLK, 
       D => buf_IPD(0),
       Q => latch_IPD(0)
     );
     latchipd1 : FD port map (
       C => buf_ICLK, 
       D => buf_IPD(1),
       Q => latch_IPD(1)
     );
     latchipd2 : FD port map (
       C => buf_ICLK, 
       D => buf_IPD(2),
       Q => latch_IPD(2)
     );
     latchipd3 : FD port map (
       C => buf_ICLK, 
       D => buf_IPD(3),
       Q => latch_IPD(3)
     );
     latchipd4 : FD port map (
       C => buf_ICLK, 
       D => buf_IPD(4),
       Q => latch_IPD(4)
     );
     latchipd5 : FD port map (
       C => buf_ICLK, 
       D => buf_IPD(5),
       Q => latch_IPD(5)
     );
     latchipd6 : FD port map (
       C => buf_ICLK, 
       D => buf_IPD(6),
       Q => latch_IPD(6)
     );
     latchipd7 : FD port map (
       C => buf_ICLK, 
       D => buf_IPD(7),
       Q => latch_IPD(7)
     );

  
end architecture imp;
